"""IP trunk product block that has all parameters of a subscription throughout its lifecycle."""

import ipaddress
from typing import TypeVar

from orchestrator.domain.base import ProductBlockModel
from orchestrator.forms.validators import UniqueConstrainedList
from orchestrator.types import SubscriptionLifecycle, strEnum

from gso.products.product_blocks.router import RouterBlock, RouterBlockInactive, RouterBlockProvisioning


class PhyPortCapacity(strEnum):
    """Physical port capacity enumerator.

    An enumerator that has the different possible capacities of ports that are available to use in subscriptions.
    """

    ONE_GIGABIT_PER_SECOND = "1G"
    TEN_GIGABIT_PER_SECOND = "10G"
    HUNDRED_GIGABIT_PER_SECOND = "100G"
    FOUR_HUNDRED_GIGABIT_PER_SECOND = "400G"


class IptrunkType(strEnum):
    DARK_FIBER = "Dark_fiber"
    LEASED = "Leased"


T = TypeVar("T", covariant=True)


class LAGMemberList(UniqueConstrainedList[T]):  # type: ignore[type-var]
    pass


class IptrunkInterfaceBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="IptrunkInterfaceBlock"
):
    #  TODO: add validation for interface names, making the type a constrained string
    interface_name: str | None = None
    interface_description: str | None = None


class IptrunkInterfaceBlockProvisioning(IptrunkInterfaceBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    interface_name: str
    interface_description: str


class IptrunkInterfaceBlock(IptrunkInterfaceBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    interface_name: str
    interface_description: str


class IptrunkSides(UniqueConstrainedList[T]):  # type: ignore[type-var]
    min_items = 2
    max_items = 2


class IptrunkSideBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="IptrunkSideBlock"
):
    iptrunk_side_node: RouterBlockInactive
    iptrunk_side_ae_iface: str | None = None
    iptrunk_side_ae_geant_a_sid: str | None = None
    iptrunk_side_ae_members: LAGMemberList[IptrunkInterfaceBlockInactive]


class IptrunkSideBlockProvisioning(IptrunkSideBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    iptrunk_side_node: RouterBlockProvisioning
    iptrunk_side_ae_iface: str | None = None
    iptrunk_side_ae_geant_a_sid: str | None = None
    iptrunk_side_ae_members: LAGMemberList[IptrunkInterfaceBlockProvisioning]


class IptrunkSideBlock(IptrunkSideBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    iptrunk_side_node: RouterBlock
    iptrunk_side_ae_iface: str | None = None
    iptrunk_side_ae_geant_a_sid: str | None = None
    iptrunk_side_ae_members: LAGMemberList[IptrunkInterfaceBlock]


class IptrunkBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="IptrunkBlock"
):
    """A trunk that's currently inactive, see :class:`IptrunkBlock`."""

    geant_s_sid: str | None = None
    iptrunk_description: str | None = None
    iptrunk_type: IptrunkType | None = None
    iptrunk_speed: PhyPortCapacity | None = None
    iptrunk_minimum_links: int | None = None
    iptrunk_isis_metric: int | None = None
    iptrunk_ipv4_network: ipaddress.IPv4Network | None = None
    iptrunk_ipv6_network: ipaddress.IPv6Network | None = None
    iptrunk_sides: IptrunkSides[IptrunkSideBlockInactive]


class IptrunkBlockProvisioning(IptrunkBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A trunk that's currently being provisioned, see :class:`IptrunkBlock`."""

    geant_s_sid: str | None = None
    iptrunk_description: str | None = None
    iptrunk_type: IptrunkType | None = None
    iptrunk_speed: PhyPortCapacity | None = None
    iptrunk_minimum_links: int | None = None
    iptrunk_isis_metric: int | None = None
    iptrunk_ipv4_network: ipaddress.IPv4Network | None = None
    iptrunk_ipv6_network: ipaddress.IPv6Network | None = None
    iptrunk_sides: IptrunkSides[IptrunkSideBlockProvisioning]


class IptrunkBlock(IptrunkBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A trunk that's currently deployed in the network."""

    #:  GÉANT service ID associated with this trunk.
    geant_s_sid: str
    #:  A human-readable description of this trunk.
    iptrunk_description: str
    #:  The type of trunk, can be either dark fibre or leased capacity.
    iptrunk_type: IptrunkType
    #:  The speed of the trunk, measured per interface associated with it.
    iptrunk_speed: PhyPortCapacity
    #:  The minimum amount of links the trunk should consist of.
    iptrunk_minimum_links: int
    #:  The :term:`IS-IS` metric of this link
    iptrunk_isis_metric: int
    #:  The IPv4 network used for this trunk.
    iptrunk_ipv4_network: ipaddress.IPv4Network
    #:  The IPv6 network used for this trunk.
    iptrunk_ipv6_network: ipaddress.IPv6Network
    #:  The two sides that the trunk is connected to.
    iptrunk_sides: IptrunkSides[IptrunkSideBlock]
